CLI interface
=============

Creating a new project
----------------------

```bash
lfa new <proj_dir>
```

Runs an interactive prompt that creates a new project in `proj_dir`.

* `-v --verbose`: Displays detailed error messages and warnings.

Developing with continuous compilation
--------------------------------------

```bash
lfa watch
```

Compiles and serves the project from memory, opens a browser window with the results, then watches your files for changes and updates the results in real time.

* `--book <proj_path>`: The path to the root of the project we're operating on. Defaults to a search in the current directory and its parents.
* `-v --verbose`: Displays detailed error messages and warnings.
* `--plugin`: Treat this project as a single plugin instead of as a book.
* `--plugin detect`: Automatically detect if this is a plugin or a book project.
* `--no-switch`: Don't ask to install lfa locally when project requires a different version of lfa. Fail instead.
* `--no-switch-check`: Continue using this version even if the project needs a different version of lfa. Might obviously fail in undefined ways.
* `-p --port <port>`: The port on which to start the server. Defaults to `$PORT` or, if that's not set, to `8080`.
* `--no-open`: Doesn't open a new browser window.
* `--bundle-name`: The name of the compiled bundle. Defaults to `book`.
* `--no-core`: Disables inclusion of the core in the bundle.
* `--no-plugins`: Disables inclusion of plugins in the bundle.
* `--no-user`: Disables inclusion of chapters and book metadata in the bundle.
* `--just-core`: Only include the core in the bundle.
* `--just-plugins`: Only include the plugins in the bundle.
* `--just-plugin <plugin_name>`: Only include a specific plugin in the bundle.
* `--just-user`: Only include chapters and book metadata in the bundle.

Building for production
-----------------------

```bash
lfa compile
```

```bash
# same as lfa compile --plugin
lfa compile-plugin 
```

Compiles the project for production. Results are saved in `.lfa/build/release`.

* `--book <proj_path>`: The path to the root of the project we're operating on. Defaults to a search in the current directory and its parents.
* `-v --verbose`: Displays detailed error messages and warnings.
* `--plugin`: Treat this project as a single plugin instead of as a book.
* `--plugin detect`: Automatically detect if this is a plugin or a book project.
* `--no-switch`: Don't ask to install lfa locally when project requires a different version of lfa. Fail instead.
* `--no-switch-check`: Continue using this version even if the project needs a different version of lfa. Might obviously fail in undefined ways.
* `--debug`: Compile a debug build instead of a production one.
* `--warnings-as-errors --Werror --werror`: Treat warnings as errors.
* `--public-path <pub_path>`: The URL where the compiled bundle will be hosted. This changes paths generated by Webpack to their absolute form. See [the Webpack docs](http://webpack.github.io/docs/configuration.html#output-publicpath).
* `--bundle-name`: The name of the compiled bundle. Defaults to `book`.
* `--no-core`: Disables inclusion of the core in the bundle.
* `--no-plugins`: Disables inclusion of plugins in the bundle.
* `--no-user`: Disables inclusion of chapters and book metadata in the bundle.
* `--just-core`: Only include the core in the bundle.
* `--just-plugins`: Only include the plugins in the bundle.
* `--just-plugin <plugin_name>`: Only include a specific plugin in the bundle.
* `--just-user`: Only include chapters and book metadata in the bundle.


Removing build products
-----------------------

```bash
lfa clean
```

```bash
# same as lfa clean --plugin
lfa clean-plugin 
```

Removes everything from the build directory (`.lfa/build`).

* `--book <proj_path>`: The path to the root of the project we're operating on. Defaults to a search in the current directory and its parents.
* `--plugin`: Treat this project as a single plugin instead of as a book.
* `--plugin detect`: Automatically detect if this is a plugin or a book project.
* `-v --verbose`: Displays detailed error messages and warnings.

